package com.fsck.k9.preferences

import app.k9mail.feature.navigation.drawer.NavigationDrawerExternalContract.DrawerConfig
import app.k9mail.legacy.preferences.ConfigManager

interface DrawerConfigManager : ConfigManager<DrawerConfig>
